/***************************************************************************
                          docking.h  -  description
                             -------------------
    begin                : Mon Jan 14 2002
    copyright            : (C) 2001, 2002 by Frank Schwanz, Ernst Martin Witte
    email                : schwanz@fh-brandenburg.de, witte@kawo1.rwth-aachen.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KRADIO_DOCKING_H
#define KRADIO_DOCKING_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <ksystemtray.h>
#include <qpixmap.h>
#include <qptrdict.h>

#include "timecontrol_interfaces.h"
#include "radio_interfaces.h"
#include "radiodevicepool_interface.h"
#include "stationselection_interfaces.h"
#include "recording-interfaces.h"
#include "plugins.h"

class RadioDocking : public KSystemTray,
                     public PluginBase,
                     public IRadioClient,
                     public ITimeControlClient,
                     public IRadioDevicePoolClient,
                     public IStationSelection,
                     public IRecordingClient
{
Q_OBJECT
public:
    RadioDocking (const QString &name);
    virtual ~RadioDocking();

    virtual bool connectI (Interface *);
    virtual bool disconnectI (Interface *);

    virtual const QString &name() const { return PluginBase::name(); }
    virtual       QString &name()       { return PluginBase::name(); }


    // PluginBase

public:
    virtual void   saveState (KConfig *) const;
    virtual void   restoreState (KConfig *);

    virtual ConfigPageInfo  createConfigurationPage();
    virtual AboutPageInfo   createAboutPage();


    // IStationSelection

RECEIVERS:
    bool setStationSelection(const QStringList &sl);

ANSWERS:
    const QStringList & getStationSelection () const { return m_stationIDs; }


    // IRadioDevicePoolClient

RECEIVERS:
    bool noticeActiveDeviceChanged(IRadioDevice *)  { return false; }
    bool noticeDevicesChanged(const QPtrList<IRadioDevice> &)  { return false; }
    bool noticeDeviceDescriptionChanged(const QString &) { return false; }

    // ITimeControlClient

RECEIVERS:
    bool noticeAlarmsChanged(const AlarmVector &)   { return false; }
    bool noticeAlarm(const Alarm &)                 { return false; }
    bool noticeNextAlarmChanged(const Alarm *);
    bool noticeCountdownStarted(const QDateTime &/*end*/);
    bool noticeCountdownStopped();
    bool noticeCountdownZero();
    bool noticeCountdownSecondsChanged(int n);


    // IRadioClient

RECEIVERS:
    bool noticePowerChanged(bool on);
    bool noticeStationChanged (const RadioStation &, int idx);
    bool noticeStationsChanged(const StationList &sl);
    bool noticePresetFileChanged(const QString &/*f*/)           { return false; }

    // IRecordingClient

RECEIVERS:
    bool noticeRecordingStarted();
    bool noticeMonitoringStarted() { return false; }
    bool noticeRecordingStopped();
    bool noticeMonitoringStopped() { return false; }
    bool noticeRecordingConfigChanged(const RecordingConfig &)   { return false; }
    bool noticeRecordingContextChanged(const RecordingContext &) { return false; }


protected slots:

    void slotSeekFwd();
    void slotSeekBkwd();

    void slotPower();
    void slotSleepCountdown();
    void slotShowAbout();

    void slotRecording();

    void slotMenuItemActivated(int id);

protected:
    void mousePressEvent( QMouseEvent *e );

    void buildContextMenu();
    void buildStationList();

    void noticeWidgetPluginShown(WidgetPluginBase *, bool shown);
    void noticePluginsChanged(const PluginList &);

    void showEvent(QShowEvent *) {}  // do nothing, original implementation adds "Quit" menu item

protected:

    KPopupMenu *m_menu;
    QStringList m_stationIDs;

    // menu Item IDs
    int            m_titleID;
    int         m_alarmID;
    int            m_recordingID;
    int            m_powerID;
    int            m_sleepID;
    int         m_seekfwID;
    int         m_seekbwID;
    QValueList<int> m_stationMenuIDs;

    QPtrDict<int>   m_widgetPluginIDs;
};


#endif
